# -*- coding: utf-8 -*-
HELPER_SETTINGS = {
    'TIME_ZONE': 'Europe/Paris',
    'INSTALLED_APPS': [
        'djangocms_admin_style'
    ],
    'TEMPLATE_CONTEXT_PROCESSORS': [
        'django.core.context_processors.debug'
    ]
}
