{% load cms_tags menu_tags sekizai_tags %}
<!DOCTYPE html>
<html>
<head>
    <title>{% block title %}This is my new project home page{% endblock title %}</title>
    {% render_block "css" %}
    <style type="text/css">
        .nav {
            padding-left: 0;
        }
        .nav li {
            display: inline;
            list-style-type: none;
            padding-right: 20px;
        }
    </style>
</head>
<body>
{% cms_toolbar %}
<div style="width: 940px; margin:0 auto">
    <ul class="nav">
        {% show_menu 0 100 100 100 %}
    </ul>
    {% block content %}
    {% endblock content %}
</div>
{% render_block "js" %}
</body>
</html>
